# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from codecs import open
from glob import iglob
from os import path as op
from subprocess import Popen, PIPE, STDOUT
import unittest as ut

import hoep as h


class MarkdownTestCase(ut.TestCase):
    def setUp(self):
        self._path = op.join(op.dirname(__file__), 'res')
        
        self.md = h.Hoep().render
    
    
    def _get_path(self, *args):
        return op.abspath(op.join(self._path, *args))
    
    def _tidy(self, html):
        args = [
            'tidy',
            '--show-body-only', '1',
            '--quiet', '1',
            '--show-warnings', '0',
            '-utf8'
        ]
        
        tidy = Popen(args, stdout = PIPE, stdin = PIPE, stderr = STDOUT)
        stdout, stderr = tidy.communicate(input = html.encode('utf-8'))
        
        return stdout.decode('utf-8')
    
    
    def runTest(self):
        for suite in ('1.0', '1.0.3'):
            for path in iglob(self._get_path('MarkdownTest_{0}'.format(suite), '*.txt')):
                test = op.splitext(path)[0]
                
                with open('{0}.txt'.format(test), 'r', encoding = 'utf-8') as f:
                    supplied = f.read()
                
                with open('{0}.html'.format(test), 'r', encoding = 'utf-8') as f:
                    expected = f.read()
                
                actual = self.md(supplied)
                
                self.assertEqual(self._tidy(actual), self._tidy(expected))



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from textwrap import dedent
import unittest as ut

import hoep as h


class _IgnoreParagraph(h.Hoep):
    def paragraph(self, text):
        return text


class BlockRenderer(h.Hoep):
    def block_code(self, text, language):
        return '[BLOCK_CODE language={1}] {0}'.format(text, language)
    
    def block_html(self, text):
        return '[BLOCK_HTML] {0}'.format(text)
    
    def block_quote(self, text):
        return '[BLOCK_QUOTE] {0}'.format(text)
    
    def header(self, text, level):
        return '[HEADER level={1}] {0}'.format(text, level)
    
    def hrule(self):
        return '[HRULE]'
    
    def list(self, text, ordered):
        return '[LIST ordered={1}]\n{0}'.format(text, ordered)
    
    def list_item(self, text, ordered):
        return '[LIST_ITEM ordered={1}] {0}'.format(text, ordered)
    
    def table(self, header, body):
        return '[TABLE]\n[HEADER]{0}\n[BODY]{1}'.format(header, body)
    
    def table_row(self, text):
        return '\n[TABLE_ROW]\n{0}'.format(text)
    
    def table_cell(self, text, flags):
        return '[TABLE_CELL text={0}]'.format(text)

class DocRenderer(_IgnoreParagraph):
    def doc_header(self):
        return 'One.\n'
    
    def doc_footer(self):
        return '\nFive.'

class FootnoteRenderer(h.Hoep):
    def footnotes(self, text):
        return '[FOOTNOTES]\n{0}'.format(text)
    
    def footnote_def(self, text, number):
        return '[FOOTNOTE_DEF number={1}] {0}'.format(text, number)
    
    def footnote_ref(self, number):
        return '[FOOTNOTE_REF number={0}]'.format(number)

class LowRenderer(_IgnoreParagraph):
    def entity(self, entity):
        return '[ENTITY] {0}'.format(entity)
    
    def normal_text(self, text):
        return '[NORMAL_TEXT] {0}'.format(text)

class ParagraphRenderer(h.Hoep):
    def paragraph(self, text):
        return '[PARAGRAPH] {0}'.format(text)

class PreRenderer(_IgnoreParagraph):
    def preprocess(self, markdown):
        return 'Nop! {0}'.format(markdown)

class PostRenderer(_IgnoreParagraph):
    def postprocess(self, html):
        return '{0} {1}'.format(html, html[::-1])

class SpanRenderer(_IgnoreParagraph):
    def autolink(self, link, email):
        return '[AUTOLINK email={1}] {0}'.format(link, email)
    
    def codespan(self, text):
        return '[CODESPAN] {0}'.format(text)
    
    def double_emphasis(self, text):
        return '[DOUBLE_EMPHASIS] {0}'.format(text)
    
    def emphasis(self, text):
        return '[EMPHASIS] {0}'.format(text)
    
    def highlight(self, text):
        return '[HIGHLIGHT] {0}'.format(text)
    
    def image(self, link, title, alt):
        return '[IMAGE link={0} title={1} alt={2}]'.format(link, title, alt)
    
    def line_break(self):
        return '[LINE_BREAK]'
    
    def link(self, link, title, content):
        return '[LINK link={0} title={1}] {2}'.format(link, title, content)
    
    def quote(self, text):
        return '[QUOTE] {0}'.format(text)
    
    def raw_html_tag(self, tag):
        return '[RAW_HTML_TAG] {0}'.format(tag)
    
    def strikethrough(self, text):
        return '[STRIKETHROUGH] {0}'.format(text)
    
    def superscript(self, text):
        return '[SUPERSCRIPT] {0}'.format(text)
    
    def triple_emphasis(self, text):
        return '[TRIPLE_EMPHASIS] {0}'.format(text)

class UnderlineRenderer(_IgnoreParagraph):
    def underline(self, text):
        return '[UNDERLINE] {0}'.format(text)


class CustomRendererTestCase(ut.TestCase):
    def setUp(self):
        self.block = BlockRenderer(h.EXT_FENCED_CODE | h.EXT_TABLES)
        self.doc = DocRenderer()
        self.footnote = FootnoteRenderer(h.EXT_FOOTNOTES)
        self.low = LowRenderer()
        self.paragraph = ParagraphRenderer()
        self.post = PostRenderer()
        self.pre = PreRenderer()
        self.span = SpanRenderer(h.EXT_AUTOLINK | h.EXT_HIGHLIGHT | h.EXT_QUOTE | h.EXT_STRIKETHROUGH | h.EXT_SUPERSCRIPT)
        self.underline = UnderlineRenderer(h.EXT_UNDERLINE)
    
    
    def md(self, md, renderer):
        return getattr(self, renderer).render(md)
    
    
    # Document level
    def test_preprocess(self):
        supplied = 'First!'
        expected = 'Nop! First!'
        
        self.assertEqual(self.md(supplied, 'pre'), expected)
    
    def test_doc_header_and_footer(self):
        supplied = 'Two.'
        expected = 'One.\nTwo.\nFive.'
        
        self.assertEqual(self.md(supplied, 'doc'), expected)
    
    def test_postprocess(self):
        supplied = 'echo'
        expected = 'echo ohce'
        
        self.assertEqual(self.md(supplied, 'post'), expected)
    
    
    # Block level
    def test_block_code(self):
        supplied = '```bash\n$ :(){ :|:& };:\n```'
        expected = '[BLOCK_CODE language=bash] $ :(){ :|:& };:\n'
        
        self.assertEqual(self.md(supplied, 'block'), expected)
    
    def test_block_html(self):
        supplied = '<p>Hi.</p>'
        expected = '[BLOCK_HTML] <p>Hi.</p>\n'
        
        self.assertEqual(self.md(supplied, 'block'), expected)
    
    def test_block_quote(self):
        supplied = '> Echo.'
        expected = '[BLOCK_QUOTE] <p>Echo.</p>\n'
        
        self.assertEqual(self.md(supplied, 'block'), expected)
    
    def test_footnotes(self):
        supplied = 'What you looking at? [^1]\n\n[^1]: Yeah, I\'m talking to you pal.'
        expected = dedent('''\
            <p>What you looking at? [FOOTNOTE_REF number=1]</p>
            [FOOTNOTES]
            [FOOTNOTE_DEF number=1] <p>Yeah, I&#39;m talking to you pal.</p>
            ''')
        
        self.assertEqual(self.md(supplied, 'footnote'), expected)
    
    def test_header(self):
        supplied = '## One more to go.'
        expected = '[HEADER level=2] One more to go.'
        
        self.assertEqual(self.md(supplied, 'block'), expected)
    
    def test_hrule(self):
        supplied = '---'
        expected = '[HRULE]'
        
        self.assertEqual(self.md(supplied, 'block'), expected)
    
    def test_list_ordered(self):
        supplied = '1. Ehh\n2. Bee\n3. Eee'
        expected = dedent('''\
            [LIST ordered=True]
            [LIST_ITEM ordered=True] Ehh
            [LIST_ITEM ordered=True] Bee
            [LIST_ITEM ordered=True] Eee
            ''')
        
        self.assertEqual(self.md(supplied, 'block'), expected)
    
    def test_list_unordered(self):
        supplied = '+ One\n+ Two\n+ Five'
        expected = dedent('''\
            [LIST ordered=False]
            [LIST_ITEM ordered=False] One
            [LIST_ITEM ordered=False] Two
            [LIST_ITEM ordered=False] Five
            ''')
        
        self.assertEqual(self.md(supplied, 'block'), expected)
    
    def test_paragraph(self):
        supplied = 'One might say this is soul sucking...'
        expected = '[PARAGRAPH] One might say this is soul sucking...'
        
        self.assertEqual(self.md(supplied, 'paragraph'), expected)
    
    def test_table(self):
        supplied = dedent(u'''\
            |  1  |  2  |  3  |
            | --- | --- | --- |
            |  X  |  X  |  O  |
            |  O  |  O  |  X  |
            |  X  |  O  |  X  |
            ''')
        expected = dedent('''\
            [TABLE]
            [HEADER]
            [TABLE_ROW]
            [TABLE_CELL text=1][TABLE_CELL text=2][TABLE_CELL text=3]
            [BODY]
            [TABLE_ROW]
            [TABLE_CELL text=X][TABLE_CELL text=X][TABLE_CELL text=O]
            [TABLE_ROW]
            [TABLE_CELL text=O][TABLE_CELL text=O][TABLE_CELL text=X]
            [TABLE_ROW]
            [TABLE_CELL text=X][TABLE_CELL text=O][TABLE_CELL text=X]''')
        
        self.assertEqual(self.md(supplied, 'block'), expected)
    
    
    # Span level
    def test_autolink(self):
        supplied = 'https://github.com/'
        expected = '[AUTOLINK email=False] https://github.com/'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_codespan(self):
        supplied = '`$ rm -Rf tests/`'
        expected = '[CODESPAN] $ rm -Rf tests/'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_double_emphasis(self):
        supplied = '__strong__'
        expected = '[DOUBLE_EMPHASIS] strong'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_emphasis(self):
        supplied = '_wat_'
        expected = '[EMPHASIS] wat'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_highlight(self):
        supplied = '==blink=='
        expected = '[HIGHLIGHT] blink'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_image(self):
        supplied = '![spacer](spacer.gif "spacer")'
        expected = '[IMAGE link=spacer.gif title=spacer alt=spacer]'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_image_ref(self):
        supplied = '![spacer][spacer]\n\n[spacer]: spacer.gif'
        expected = '[IMAGE link=spacer.gif title=None alt=spacer]'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_line_break(self):
        supplied = 'So.  \nTired.'
        expected = 'So.[LINE_BREAK]Tired.'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_link(self):
        supplied = '[GitHub](https://github.com/)'
        expected = '[LINK link=https://github.com/ title=None] GitHub'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_link_ref(self):
        supplied = '[GitHub][github]\n\n[github]: https://github.com/ "GitHub"'
        expected = '[LINK link=https://github.com/ title=GitHub] GitHub'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_quote(self):
        supplied = '"Air quotes are obnoxious."'
        expected = '[QUOTE] Air quotes are obnoxious.'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_raw_html_tag(self):
        supplied = '<halp/>'
        expected = '[RAW_HTML_TAG] <halp/>'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_strikethrough(self):
        supplied = 'I\'m ~~running~~ out of ideas.'
        expected = 'I&#39;m [STRIKETHROUGH] running out of ideas.'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_superscript(self):
        supplied = '^bro'
        expected = '[SUPERSCRIPT] bro'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_triple_emphasis(self):
        supplied = 'Triple emphasis? That\'s ___absurd___.'
        expected = 'Triple emphasis? That&#39;s [TRIPLE_EMPHASIS] absurd.'
        
        self.assertEqual(self.md(supplied, 'span'), expected)
    
    def test_underline(self):
        supplied = 'That\'s _it_?'
        expected = 'That&#39;s [UNDERLINE] it?'
        
        self.assertEqual(self.md(supplied, 'underline'), expected)
    
    
    # Low-level
    def test_low_level(self):
        supplied = '&#9731;'
        expected = '[NORMAL_TEXT] [ENTITY] &#9731;'
        
        self.assertEqual(self.md(supplied, 'low'), expected)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import unittest as ut

import hoep as h


class SmartyPantsTestCase(ut.TestCase):
    def setUp(self):
        self.md = h.Hoep(render_flags = h.HTML_SMARTYPANTS).render
    
    
    def test_apostrophe(self):
        supplied = 'What\'s with apostrophes?'
        expected = '<p>What&rsquo;s with apostrophes?</p>\n'
        
        self.assertEqual(self.md(supplied), expected)
    
    def test_double_quotes(self):
        supplied = '"Air quotes are obnoxious."'
        expected = '<p>&ldquo;Air quotes are obnoxious.&rdquo;</p>\n'
        
        self.assertEqual(self.md(supplied), expected)
    
    def test_ellipsis(self):
        supplied = 'One of these days...'
        expected = '<p>One of these days&hellip;</p>\n'
        
        self.assertEqual(self.md(supplied), expected)
    
    def test_em_dash(self):
        supplied = 'In five minutes the---'
        expected = '<p>In five minutes the&mdash;</p>\n'
        
        self.assertEqual(self.md(supplied), expected)
    
    def test_en_dash(self):
        supplied = 'Non--zero.'
        expected = '<p>Non&ndash;zero.</p>\n'
        
        self.assertEqual(self.md(supplied), expected)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from codecs import open
from os import path as op
import unittest as ut

import hoep as h


class UnicodeTestCase(ut.TestCase):
    def setUp(self):
        self._path = op.join(op.dirname(__file__), 'res')
        
        self.md = h.Hoep().render
    
    
    def _get_path(self, *args):
        return op.abspath(op.join(self._path, *args))
    
    
    def runTest(self):
        with open(self._get_path('Unicode', 'unicode.txt'), 'r', encoding = 'utf-8') as f:
            supplied = f.read()
        
        with open(self._get_path('Unicode', 'unicode.html'), 'r', encoding = 'utf-8') as f:
            expected = f.read()
        
        self.assertEqual(self.md(supplied), expected)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from textwrap import dedent
import unittest as ut

import hoep as h


class ExtensionsTestCase(ut.TestCase):
    def setUp(self):
        self.extensions = {
            'autolink': h.EXT_AUTOLINK,
            'fenced_code': h.EXT_FENCED_CODE,
            'footnotes': h.EXT_FOOTNOTES,
            'highlight': h.EXT_HIGHLIGHT,
            'lax_spacing': h.EXT_LAX_SPACING,
            'no_indented_code': h.EXT_DISABLE_INDENTED_CODE,
            'no_intra_emphasis': h.EXT_NO_INTRA_EMPHASIS,
            'quote': h.EXT_QUOTE,
            'space_headers': h.EXT_SPACE_HEADERS,
            'strikethrough': h.EXT_STRIKETHROUGH,
            'superscript': h.EXT_SUPERSCRIPT,
            'tables': h.EXT_TABLES,
            'underline': h.EXT_UNDERLINE
        }
    
    
    def md(self, md, extension):
        return h.render(md, self.extensions[extension])
    
    
    def test_autolink(self):
        supplied = 'https://github.com/'
        expected = '<p><a href="https://github.com/">https://github.com/</a></p>\n'
        
        self.assertEqual(self.md(supplied, 'autolink'), expected)
    
    def test_fenced_code(self):
        supplied = '```\n$ :(){ :|:& };:\n```'
        expected = '<pre><code>$ :(){ :|:&amp; };:\n</code></pre>\n'
        
        self.assertEqual(self.md(supplied, 'fenced_code'), expected)
    
    def test_fenced_code_lang(self):
        supplied = '```bash\n$ :(){ :|:& };:\n```'
        expected = '<pre><code class="language-bash">$ :(){ :|:&amp; };:\n</code></pre>\n'
        
        self.assertEqual(self.md(supplied, 'fenced_code'), expected)
    
    def test_footnotes(self):
        supplied = 'What you looking at? [^1]\n\n[^1]: Yeah, I\'m talking to you pal.'
        expected = dedent('''\
            <p>What you looking at? <sup id="fnref1"><a href="#fn1" rel="footnote">1</a></sup></p>
            
            <div class="footnotes">
            <hr>
            <ol>
            
            <li id="fn1">
            <p>Yeah, I&#39;m talking to you pal.&nbsp;<a href="#fnref1" rev="footnote">&#8617;</a></p>
            </li>
            
            </ol>
            </div>
            ''')
        
        self.assertEqual(self.md(supplied, 'footnotes'), expected)
    
    def test_highlight(self):
        supplied = 'I\'m ==special==.'
        expected = '<p>I&#39;m <mark>special</mark>.</p>\n'
        
        self.assertEqual(self.md(supplied, 'highlight'), expected)
    
    def test_lax_spacing(self):
        supplied = 'This extension is madness.\n<p>Truly.</p>\n'
        expected = '<p>This extension is madness.</p>\n\n<p>Truly.</p>\n'
        
        self.assertEqual(self.md(supplied, 'lax_spacing'), expected)
    
    def test_no_indented_code(self):
        supplied = '    $ :(){ :|:& };:'
        expected = '<p>$ :(){ :|:&amp; };:</p>\n'
        
        self.assertEqual(self.md(supplied, 'no_indented_code'), expected)
    
    def test_no_intra_emphasis(self):
        supplied = 'Sorry,_my_space_key_is_broken.'
        expected = '<p>Sorry,_my_space_key_is_broken.</p>\n'
        
        self.assertEqual(self.md(supplied, 'no_intra_emphasis'), expected)
    
    def test_quote(self):
        supplied = '"Air quotes are obnoxious."'
        expected = '<p><q>Air quotes are obnoxious.</q></p>\n'
        
        self.assertEqual(self.md(supplied, 'quote'), expected)
    
    def test_space_headers(self):
        supplied = '#Are you listening to me?!'
        expected = '<p>#Are you listening to me?!</p>\n'
        
        self.assertEqual(self.md(supplied, 'space_headers'), expected)
    
    def test_strikethrough(self):
        supplied = 'I\'m ~~running~~ out of ideas.'
        expected = '<p>I&#39;m <del>running</del> out of ideas.</p>\n'
        
        self.assertEqual(self.md(supplied, 'strikethrough'), expected)
    
    def test_superscript(self):
        supplied = '^bro'
        expected = '<p><sup>bro</sup></p>\n'
        
        self.assertEqual(self.md(supplied, 'superscript'), expected)
    
    def test_tables(self):
        supplied = dedent(u'''\
            |  1  |  2  |  3  |
            | --- | --- | --- |
            |  X  |  X  |  O  |
            |  O  |  O  |  X  |
            |  X  |  O  |  X  |
            ''')
        expected = dedent('''\
            <table><thead>
            <tr>
            <th>1</th>
            <th>2</th>
            <th>3</th>
            </tr>
            </thead><tbody>
            <tr>
            <td>X</td>
            <td>X</td>
            <td>O</td>
            </tr>
            <tr>
            <td>O</td>
            <td>O</td>
            <td>X</td>
            </tr>
            <tr>
            <td>X</td>
            <td>O</td>
            <td>X</td>
            </tr>
            </tbody></table>
            ''')
        
        self.assertEqual(self.md(supplied, 'tables'), expected)
    
    def test_underline(self):
        supplied = 'What do you _mean_?'
        expected = '<p>What do you <u>mean</u>?</p>\n'
        
        self.assertEqual(self.md(supplied, 'underline'), expected)



'''
Hoep
====

Hoep is a Python binding for the Hoedown Markdown library.


Install
-------

Hoep is available on PyPi so installation is only a quick ``pip`` away.

::

    $ pip install hoep


Unicode
-------

Hoep only accepts and returns ``unicode`` objects in Python 2 and ``str`` objects in Python 3. This goes for custom renderers as well. All callbacks receive and must return ``unicode`` or ``str`` objects, respective to the Python version.


Basic Usage
-----------

Using Hoep is pretty straightforward. You can either use the ``render`` module function or the ``Hoep`` module class. The latter is more efficient if you are rendering multiple documents in a single go.

.. code:: python

    # -*- coding: utf-8 -*-

    from __future__ import unicode_literals

    import hoep as h


    extensions = h.EXT_STRIKETHROUGH
    render_flags = h.HTML_SMARTYPANTS


    # Method form.
    html = h.render('Some ~~bold~~ __"strong"__ text.', extensions, render_flags)


    # Class form.
    md = h.Hoep(extensions, render_flags)

    html = md.render('Some ~~bold~~ __"strong"__ text.')

Extensions
~~~~~~~~~~

You can enable multiple extensions by bitwise ORing them.

-  ``EXT_AUTOLINK``: Automatically parse URLs into links.
-  ``EXT_DISABLE_INDENTED_CODE``: Disables indented code blocks.
-  ``EXT_FENCED_CODE``: Enables fenced code blocks.
-  ``EXT_FOOTNOTES``: Enables `Markdown Extra style footnotes`_.
-  ``EXT_HIGHLIGHT``: Enables ==marking== text.
-  ``EXT_LAX_SPACING``: Removes the need for an empty line between Markdown and raw HTML.
-  ``EXT_NO_INTRA_EMPHASIS``: Disables emphasis\_between\_words.
-  ``EXT_QUOTE``: "Quotes" are translated into ``<q>`` tags.
-  ``EXT_SPACE_HEADERS``: ATX style headers require a space after the opening number sign(s).
-  ``EXT_STRIKETHROUGH``: Enables ~~striking~~ text.
-  ``EXT_SUPERSCRIPT``: Enables super^script.
-  ``EXT_TABLES``: Enables `Markdown Extra style tables`_.
-  ``EXT_UNDERLINE``: Translates ``<em>`` tags into ``<u>`` tags.

Render flags
~~~~~~~~~~~~

You can enable multiple render flags by bitwise ORing them.

-  ``HTML_ESCAPE``: All HTML is escaped.
-  ``HTML_EXPAND_TABS``: Tabs are expanded to spaces.
-  ``HTML_HARD_WRAP``: Line breaks are translated into ``<br>`` tags.
-  ``HTML_SAFELINK``: Only links to safe protocols are allowed.
-  ``HTML_SKIP_HTML``: All HTML tags are stripped.
-  ``HTML_SKIP_IMAGES``: Images are ignored.
-  ``HTML_SKIP_LINKS``: Links are ignored.
-  ``HTML_SKIP_STYLE``: ``<style>`` tags are stripped.
-  ``HTML_SMARTYPANTS``: Enables SmartyPants.
-  ``HTML_TOC``: Anchors are added to headers.
-  ``HTML_USE_XHTML``: Renders XHTML instead of HTML.


Custom renderer
---------------

By subclassing the ``Hoep`` class, you can create a custom renderer overriding various callbacks.

.. code:: python

    # -*- coding: utf-8 -*-

    from __future__ import unicode_literals

    import hoep as h


    class MyRenderer(h.Hoep):
        def __init__(self, extensions = 0, render_flags = 0):
            # If you override the `__init__` function, you must call
            # the parent class's or bad things will happen.
            super(MyRenderer, self).__init__(extensions, render_flags)
            
            self.replace = '[redacted]'
        
        
        def strikethrough(self, text):
            return self.replace


    md = MyRenderer(h.EXT_STRIKETHROUGH)

    html = md.render('Some ~~bold~~ __"strong"__ text.')

Callbacks
~~~~~~~~~

Document level
^^^^^^^^^^^^^^

+  ``preprocess(str markdown)``
+  ``doc_header()``
+  ``doc_footer()``
+  ``postprocess(str html)``

Block level
^^^^^^^^^^^

+  ``block_code(str text, str language)``
+  ``block_html(str text)``
+  ``block_quote(str text)``
+  ``footnotes(str text)``
+  ``footnote_def(str text, int number)``
+  ``header(str text, int level)``
+  ``hrule()``
+  ``list(str text, bool ordered)``
+  ``list_item(str text, bool ordered)``
+  ``paragraph(str text)``
+  ``table(str header, str body)``
+  ``table_row(str text)``
+  ``table_cell(str text, int flags)``

Span level
^^^^^^^^^^

+  ``autolink(str link, bool is_email)``
+  ``codespan(str text)``
+  ``double_emphasis(str text)``
+  ``emphasis(str text)``
+  ``footnote_ref(int number)``
+  ``highlight(str text)``
+  ``image(str link, mixed title, str alt)``
+  ``line_break()``
+  ``link(str link, mixed title, str content)``
+  ``quote(str text)``
+  ``raw_html_tag(str tag)``
+  ``strikethrough(str text)``
+  ``superscript(str text)``
+  ``triple_emphasis(str text)``
+  ``underline(str text)``

Low-level
^^^^^^^^^

+  ``entity(str entity)``
+  ``normal_text(str text)``

Support
-------

If you run into any issues or have any questions, either open an `issue`_ or hop in `#mynt`_ on irc.freenode.net.


.. _#mynt: http://webchat.freenode.net/?channels=mynt
.. _issue: https://github.com/Anomareh/Hoep/issues
.. _Markdown Extra style footnotes: http://michelf.ca/projects/php-markdown/extra/#footnotes
.. _Markdown Extra style tables: http://michelf.ca/projects/php-markdown/extra/#table
'''
from sys import version_info

from setuptools import Extension, setup


_version = '2' if version_info[0] == 2 else '3'
_sources = [
    'ext/py{0}k/hoep.c'.format(_version),
    'ext/py{0}k/wrapper.c'.format(_version),
    'ext/hoedown/autolink.c',
    'ext/hoedown/buffer.c',
    'ext/hoedown/document.c',
    'ext/hoedown/escape.c',
    'ext/hoedown/html.c',
    'ext/hoedown/html_blocks.c',
    'ext/hoedown/html_smartypants.c',
    'ext/hoedown/stack.c',
    'ext/hoedown/version.c'
]


setup(
    name = 'hoep',
    version = '1.0',
    author = 'Andrew Fricke',
    author_email = 'andrew@uhnomoli.com',
    url = 'https://github.com/Anomareh/Hoep',
    description = 'A Python binding for the Hoedown Markdown library.',
    long_description = __doc__,
    license = 'BSD',
    platforms = 'any',
    zip_safe = False,
    ext_modules = [
        Extension(
            'hoep',
            sources = _sources
        )
    ],
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: C',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Text Processing',
        'Topic :: Utilities'
    ]
)
